@@ -25,30 +25,18 @@ OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 /* Macintosh Gestalt interface */
 
 #include "Python.h"
-#include "macglue.h"
+#include "pymactoolbox.h"
 
-#ifdef WITHOUT_FRAMEWORKS
-#include <Types.h>
-#include <Gestalt.h>
-#else
 #include <Carbon/Carbon.h>
-#endif
 
 static PyObject *
 gestalt_gestalt(PyObject *self, PyObject *args)
 {
 	OSErr iErr;
-	char *str;
-	int size;
 	OSType selector;
 	long response;
-	if (!PyArg_Parse(args, "s#", &str, &size))
+	if (!PyArg_Parse(args, "O&", PyMac_GetOSType, &selector))
 		return NULL;
-	if (size != 4) {
-		PyErr_SetString(PyExc_TypeError, "gestalt arg must be 4-char string");
-		return NULL;
-	}
-	selector = *(OSType*)str;
 	iErr = Gestalt ( selector, &response );
 	if (iErr != 0) 
 		return PyMac_Error(iErr);
