@@ -1,35 +1,9 @@
-/***********************************************************
-Copyright 1991-1995 by Stichting Mathematisch Centrum, Amsterdam,
-The Netherlands.
 
-                        All Rights Reserved
+#include "Python.h"
 
-Permission to use, copy, modify, and distribute this software and its
-documentation for any purpose and without fee is hereby granted,
-provided that the above copyright notice appear in all copies and that
-both that copyright notice and this permission notice appear in
-supporting documentation, and that the names of Stichting Mathematisch
-Centrum or CWI or Corporation for National Research Initiatives or
-CNRI not be used in advertising or publicity pertaining to
-distribution of the software without specific, written prior
-permission.
-
-While CWI is the initial source for this software, a modified version
-is made available by the Corporation for National Research Initiatives
-(CNRI) at the Internet address ftp://ftp.python.org.
-
-STICHTING MATHEMATISCH CENTRUM AND CNRI DISCLAIM ALL WARRANTIES WITH
-REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF
-MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL STICHTING MATHEMATISCH
-CENTRUM OR CNRI BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
-DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
-PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
-TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
-PERFORMANCE OF THIS SOFTWARE.
-
-******************************************************************/
-
-#include "config.h"
+#if defined(__sgi) && defined(WITH_THREAD) && !defined(_SGI_MP_SOURCE)
+#define _SGI_MP_SOURCE
+#endif
 
 /* Convert a possibly signed character to a nonnegative int */
 /* XXX This assumes characters are 8 bits wide */
@@ -39,8 +13,6 @@ PERFORMANCE OF THIS SOFTWARE.
 #define Py_CHARMASK(c)		((c) & 0xff)
 #endif
 
-#include "rename2.h"
-
 /* strtol and strtoul, renamed to avoid conflicts */
 
 /*
@@ -58,13 +30,12 @@ PERFORMANCE OF THIS SOFTWARE.
 */
 
 #include <ctype.h>
+#ifndef DONT_HAVE_ERRNO_H
 #include <errno.h>
+#endif
 
 unsigned long
-mystrtoul(str, ptr, base)
-register char *	str;
-char **		ptr;
-int		base;
+PyOS_strtoul(register char *str, char **ptr, int base)
 {
     register unsigned long	result;	/* return value of the function */
     register int		c;	/* current input character */
@@ -112,7 +83,7 @@ int		base;
     }
 
 /* do the conversion */
-    while (c = Py_CHARMASK(*str))
+    while ((c = Py_CHARMASK(*str)) != '\0')
     {
 	if (isdigit(c) && c - '0' < base)
 	    c -= '0';
@@ -129,10 +100,14 @@ int		base;
 	}
 	temp = result;
 	result = result * base + c;
-#ifndef MPW
-	if ((result - c) / base != temp)	/* overflow */
-	    ovf = 1;
-#endif
+	if(base == 10) {
+		if(((long)(result - c) / base != (long)temp))	/* overflow */
+			ovf = 1;
+	}
+	else {
+		if ((result - c) / base != temp)	/* overflow */
+			ovf = 1;
+	}
 	str++;
     }
 
@@ -141,39 +116,44 @@ int		base;
 	*ptr = str;
     if (ovf)
     {
-	result = ~0;
+	result = (unsigned long) ~0L;
 	errno = ERANGE;
     }
     return result;
 }
 
+/* Checking for overflow in PyOS_strtol is a PITA; see comments
+ * about Py_ABS_LONG_MIN in longobject.c.
+ */
+#define Py_ABS_LONG_MIN		(0-(unsigned long)LONG_MIN)
+
 long
-mystrtol(str, ptr, base)
-char *	str;
-char ** ptr;
-int	base;
+PyOS_strtol(char *str, char **ptr, int base)
 {
 	long result;
+	unsigned long uresult;
 	char sign;
-	
+
 	while (*str && isspace(Py_CHARMASK(*str)))
 		str++;
-	
+
 	sign = *str;
 	if (sign == '+' || sign == '-')
 		str++;
-	
-	result = (long) mystrtoul(str, ptr, base);
-	
-	/* Signal overflow if the result appears negative,
-	   except for the largest negative integer */
-	if (result < 0 && !(sign == '-' && result == -result)) {
+
+	uresult = PyOS_strtoul(str, ptr, base);
+
+	if (uresult <= (unsigned long)LONG_MAX) {
+		result = (long)uresult;
+		if (sign == '-')
+			result = -result;
+	}
+	else if (sign == '-' && uresult == Py_ABS_LONG_MIN) {
+		result = LONG_MIN;
+	}
+	else {
 		errno = ERANGE;
 		result = 0x7fffffff;
 	}
-	
-	if (sign == '-')
-		result = -result;
-	
 	return result;
 }
